/*
  repeater.hpp

  Copyright (C) 2011 Taiji Yamada <taiji@aihara.co.jp>
*/
#ifndef _REPEATER_HPP_
#define _REPEATER_HPP_
#include <sstream>
#include <string>
#include <boost/optional.hpp>

template<typename T, typename D>
struct repeater {
  T front, back;
  D diff;
  repeater() : front(), back(), diff() {}
  repeater(T s) : front(s), back(), diff() {}
  repeater(T s, T e) : front(s), back(e), diff(1) {}
  repeater(T s, D d, T e) : front(s), back(e), diff(d) {}
  const bool is_repeat() const { return (diff != D()); }
  const bool is_unlimited() const { return (is_repeat() && (front == back)); }
  const long count() const { return long((!is_repeat()) ? 1 : (!is_unlimited()) ? ((back - front)/diff + 1) : 0); }
  const T last() const { return (!is_repeat()) ? front : (!is_unlimited()) ? front + diff*(count() - 1) : T(); }
  const boost::optional<T> before(const T &a) const
  {
    return (!(a < front)) ? boost::optional<T>((!is_repeat()) ? front : (last() < a) ? last() :
					       front + diff*((a - front)/diff)) : boost::optional<T>();
  }
  const boost::optional<T> after(const T &a) const
  {
    return (!(last() < a)) ? boost::optional<T>((!is_repeat()) ? front : (a < front) ? front :
						(is_unlimited()) ? front + diff*((a - front + diff - D(1))/diff) :
						last() - diff*((last() - a)/diff)) : boost::optional<T>();
  }
};

/*
  0,0,*:	0		is_repeat() == false
  0,1,0:	0~		is_repeat() && is_unlimited() == true
  0,1,!0:	0~!0		is_repeat() == true
  0,2,0:	0~2~		is_repeat() && is_unlimited() == true
  0,2,!0:	0~2~!0		is_repeat() == true
 */
template<typename T, typename D>
inline
std::ostream &operator<<(std::ostream &os, const repeater<T, D> &a)
{
  const char separator = '~';
  os << a.front;
  if (a.diff != D())
    if (a.diff == D(1)) {
      os << separator;
      if (a.front != a.back)
	os << a.back;
    }
    else {
      os << separator << a.diff << separator;
      if (a.front != a.back)
	os << a.back;
    }
  return os;
}

template<typename T, typename D>
inline
std::istream &operator>>(std::istream &is, repeater<T, D> &a)
{
  const std::string separators(".~"), spaces("\t\r\n ");
  int i = 0, j = 0;
  std::stringstream ss[3];
  char c;
  a.front = a.back = T();
  a.diff = D();
  do {
    is >> c;
    if (!is.good()) { is.clear(); break; }
    if (separators.find(c) != std::string::npos) ++i;
    else if (spaces.find(c) != std::string::npos) ;
    else ss[j=i] << c;
  } while (j < 3);
  ++j;
  ss[0] >> a.front;
  if (i != j) {
    if (j == 2) {
      ss[1] >> a.back;
      a.diff = D(1);
    }
    else {
      ss[1] >> a.diff;
      ss[2] >> a.back;
    }
  }
  else {
    if (j == 1)
      a.diff = D(1);
    else
      ss[1] >> a.diff;
    a.back = a.front;
  }
  return is;
}
#endif
